#include <linux/init.h>
#include <linux/module.h>
#include <linux/types.h>

#include "cmdparser.h"

struct parse_result* parse_command(const char* cmd)
{

  unsigned int parse_reg_ok = 0;

  static struct parse_result parse_result;

  printk(KERN_DEBUG "CMD length:%lu\n", (unsigned long)strlen(cmd));

  //- reset result struct members
  parse_result.cr_register = 'x';
  parse_result.cmd = "xx";
  parse_result.cmd_parameter_bit = 0;
  parse_result.cmd_parameter_hex = 0;
  parse_result.status = 1;

  //- parse command
  if (cmd[0] != 'c' && cmd[1] != 'r') {
    printk(KERN_INFO "CMD Parser failed for:%s\n", cmd);
    parse_result.status = 1;
  }

  //- check register, always do a strlen check!
  if (cmd[0] == 'c' && cmd[1] == 'r' && strlen(cmd) >=3) {
    if (cmd[2] == '0' || cmd[2] == '2' || cmd[2] == '3' || cmd[2] == '4' || cmd[2] == '8') {
      parse_result.cr_register = cmd[2];
      parse_reg_ok = 1;
    }
  }

  //- check command, always do a strlen check!
  if (parse_reg_ok == 1 && strlen(cmd) >=6) {

    printk(KERN_DEBUG "CMD [4]:%c [5]:%c\n", cmd[4], cmd[5]);

    if (cmd[4] == 'B' && cmd[5] == 's') {
      parse_result.cmd = (const char*)"Bs";
    }
    if (cmd[4] == 'B' && cmd[5] == 'c') {
      parse_result.cmd = (const char*)"Bc";
    }
    if (cmd[4] == 'B' && cmd[5] == 't') {
      parse_result.cmd = (const char*)"Bt";
    }
    if (cmd[4] == 'H' && cmd[5] == 's') {
      parse_result.cmd = (const char*)"Hs";
    }
  }

  //- check bit operation and bit operation parameter
  if (parse_reg_ok == 1 && cmd[4] == 'B' && strlen(cmd) == 10) {

    unsigned int digit1 = parse_digit(cmd[7]);
    unsigned int digit2 = parse_digit(cmd[8]);

    parse_result.cmd_parameter_bit = (digit1*10)+digit2;

    if (parse_result.cmd_parameter_bit >= 0 && parse_result.cmd_parameter_bit <= 32) {
      parse_result.status = 0;
    }

  }

  //- check hex operation and paramter
  if (parse_reg_ok == 1 && cmd[4] == 'H' && strlen(cmd) == 18) {

    unsigned int digit1 = parse_digit_hex(cmd[9]);
    unsigned int digit2 = parse_digit_hex(cmd[10]);
    unsigned int digit3 = parse_digit_hex(cmd[11]);
    unsigned int digit4 = parse_digit_hex(cmd[12]);
    unsigned int digit5 = parse_digit_hex(cmd[13]);
    unsigned int digit6 = parse_digit_hex(cmd[14]);
    unsigned int digit7 = parse_digit_hex(cmd[15]);
    unsigned int digit8 = parse_digit_hex(cmd[16]);

    unsigned int res_byte1 = ((digit1*16)+digit2)*16*16*16*16*16*16;
    unsigned int res_byte2 = ((digit3*16)+digit4)*16*16*16*16;
    unsigned int res_byte3 = ((digit5*16)+digit6)*16*16;
    unsigned int res_byte4 = ((digit7*16)+digit8);

    printk(KERN_DEBUG "DIGITS 1:%d 2:%d 3:%d 4:%d 5:%d 6:%d 7:%d 8:%d\n", digit1, digit2, digit3, digit4, digit5, digit6, digit7, digit8);

    parse_result.cmd_parameter_hex = res_byte1 + res_byte2 + res_byte3 + res_byte4;

    parse_result.status = 0;

  }


  printk(KERN_DEBUG "CMD Result - register:%c cmd:%s param_bit:%d param_hex:%lu status:%d\n", parse_result.cr_register, parse_result.cmd, parse_result.cmd_parameter_bit, parse_result.cmd_parameter_hex, parse_result.status);

  return &parse_result;
}

//- parse ascii decimal digit and return integer value
unsigned int parse_digit(char ascii_digit) {

  unsigned int digit = 0;

  if (ascii_digit == '0') { digit = 0; }
  if (ascii_digit == '1') { digit = 1; }
  if (ascii_digit == '2') { digit = 2; }
  if (ascii_digit == '3') { digit = 3; }
  if (ascii_digit == '4') { digit = 4; }
  if (ascii_digit == '5') { digit = 5; }
  if (ascii_digit == '6') { digit = 6; }
  if (ascii_digit == '7') { digit = 7; }
  if (ascii_digit == '8') { digit = 8; }
  if (ascii_digit == '9') { digit = 9; }

  return digit;
}

//- parse ascii hex digit and return integer value
unsigned int parse_digit_hex(char ascii_digit) {

  unsigned int digit = 0;

  if (ascii_digit == '0') { digit = 0; }
  if (ascii_digit == '1') { digit = 1; }
  if (ascii_digit == '2') { digit = 2; }
  if (ascii_digit == '3') { digit = 3; }
  if (ascii_digit == '4') { digit = 4; }
  if (ascii_digit == '5') { digit = 5; }
  if (ascii_digit == '6') { digit = 6; }
  if (ascii_digit == '7') { digit = 7; }
  if (ascii_digit == '8') { digit = 8; }
  if (ascii_digit == '9') { digit = 9; }
  if (ascii_digit == 'a') { digit = 10; }
  if (ascii_digit == 'b') { digit = 11; }
  if (ascii_digit == 'c') { digit = 12; }
  if (ascii_digit == 'd') { digit = 13; }
  if (ascii_digit == 'e') { digit = 14; }
  if (ascii_digit == 'f') { digit = 15; }

  return digit;
}

EXPORT_SYMBOL(parse_command);
