#include <linux/init.h>
#include <linux/module.h>
#include <linux/types.h>

#include "dumpbits.h"

char* dumpbits_byte(const char in_byte) {

    unsigned char mask = 0x80; // bit:10000000

    int bit_set_unset = 120;

    static char status_bit_chars[8] = "xxxxxxxx";

    for(int x=0; x<8; x++) {
      bit_set_unset = mask & in_byte ? 49 : 48;
      status_bit_chars[x] = bit_set_unset;
      mask = mask >> 1;
    }

    return status_bit_chars;
}

struct bit_status* dumpbits(void *var_pointer, unsigned int size_var) {

    //- cast void pointer to char pointer (1 byte)
    char *P1 = (char*)var_pointer;

    //- setup status struct (static, because returning pointer to it)
    static struct bit_status bit_stat;

    //- reset status bit chars
    char *status_bit_chars = "xxxxxxxx";

    //- set position for x86_64 little endianess decrement
    unsigned int pos = (size_var-1);

    //- init struct vars
    bit_stat.bit_status_byte[0] = 0;
    bit_stat.bit_status_byte[1] = 0;
    bit_stat.bit_status_byte[2] = 0;
    bit_stat.bit_status_byte[3] = 0;

    for (unsigned int x=0; x<BIT_STRING_LENGTH; ++x) {
        bit_stat.bit_status_string[x] = 0;
    }  

    //- loop on sizeof() given size_var
    for (unsigned int x=0; x<size_var; ++x) {

        //- write dump single byte into status struct
        bit_stat.bit_status_byte[pos] = *P1;

        //- process bits of passed byte pointer
        status_bit_chars = dumpbits_byte(*P1);

        //- write result bits (unsigned char) into status struct
        bit_stat.bit_status_string[(pos*9)+1] = status_bit_chars[0];
        bit_stat.bit_status_string[(pos*9)+2] = status_bit_chars[1];
        bit_stat.bit_status_string[(pos*9)+3] = status_bit_chars[2];
        bit_stat.bit_status_string[(pos*9)+4] = status_bit_chars[3];
        bit_stat.bit_status_string[(pos*9)+5] = status_bit_chars[4];
        bit_stat.bit_status_string[(pos*9)+6] = status_bit_chars[5];
        bit_stat.bit_status_string[(pos*9)+7] = status_bit_chars[6];
        bit_stat.bit_status_string[(pos*9)+8] = status_bit_chars[7];

        //- write 8bit seperator (space)
        bit_stat.bit_status_string[(pos*9)] = 32;

        //- increment var pointer (+1byte)
        ++P1;

        //- decrement pos
        --pos;
    }

    //- debug output;
    printk(KERN_DEBUG "dumpbits struct status bytes:%02x%02x%02x%02x\n", bit_stat.bit_status_byte[0], bit_stat.bit_status_byte[1], bit_stat.bit_status_byte[2], bit_stat.bit_status_byte[3]);
    printk(KERN_DEBUG "dumpbits struct status string:%s\n", bit_stat.bit_status_string);

    return &bit_stat;
}

EXPORT_SYMBOL(dumpbits);
EXPORT_SYMBOL(dumpbits_byte);
